//Hex/RGB colour code converter, made and maintained by Mobius Evalon (mobiusevalon@tibbius.com). \
Credit isn't required but greatly appreciated.

//Anywhere a hexadecimal code is required, you are able to use the "full" seven-character form \
(such as #ff0000) or the "shorthand" four-character form (such as #f00), and anywhere an RGB code \
is required, a text string with any non-number delimiter (such as 255,0,0 where the comma is the \
delimiter) or a list of numbers are both valid. In both the hrc_ishex() and hrc_isrgb() procs, \
the integer 1 is returned when the value is valid, and the integer 0 is returned when an error \
is encountered anywhere in the library.

//hrc_hex2rgb(hex code,return list = 0) \
The function of hrc_hex2rgb is rather simple, converting a hexadecimal in either the "full" \
seven-character form or the "shorthand" four-character form to an RGB value. The default \
behaviour of the proc is to return a comma-delimited text string (such as 255,0,0), but by \
giving the second argument a true value, you recieve a list with a length of three instead \
(containing the red, green and blue values in that order).
mob/verb/give_rgb(t as text)
	var/rgb = hrc_hex2rgb(t)
	if(!t) src << "\"[t]\" is not a valid hexadecimal code."
	else src << "\"[t]\" as an RGB code is \"[rgb]\"."

//The uses of the second argument to return a list are many, including, for example, if you just \
want the green value in the RGB code; it's a lot easier to extract values from a list than to \
copytext them out of a string.
mob/verb/give_green(t as text)
	var/rgb[] = hrc_hex2rgb(t,1)
	if(!t) src << "\"[t]\" is not a valid hexadecimal code."
	else src << "The green value is \"[rgb[2]]\"."

//hrc_invert(hex or RGB code) \
This proc returns the inverted value of the argument given to it (black turns white, \
yellow turns blue, etc). It returns the inverted value in the same form it was given; a list of \
RGB numbers returns a list of RGB numbers, a hexadecimal value returns a hexadecimal value, etc.
mob/verb/invert(t as text)
	var/i = hrc_invert(t)
	if(!i) src << "\"[t]\" is not a valid colour code."
	else src << "The inverted value of \"[t]\" is [i]."

//hrc_ishex(hex code) \
hrc_ishex() simply returns 1 or 0 depending on whether the given value is a valid hexadecimal \
code. It could be useful if, for example, you allow players to enter their own hexadecimal code \
for their name colour.
mob/var/name_colour = "#ff0000"

mob/verb/change_name_colour(t as text)
	var/v = hrc_ishex(t)
	if(!v) src << "\"[t]\" is not a valid hexadecimal code!"
	else src.name_colour = t

mob/verb/say(t as text)
	if(t) world << "<span style=\"color: [src.name_colour]\">[src.name]</span>: <span style=\"color: [src.text_colour]\">[t]</span>"

//hrc_isrgb(RGB code) \
Returns 1 or 0 depending on whether or not the provided value is a valid RGB decimal code. \
You could use this in conjunction with hrc_ishex() to allow input of RGB decimal values in \
tandem with hexadecimal codes.
mob/var/text_colour = "#0000ff"

mob/verb/change_text_colour(t as text)
	if(hrc_ishex(t) || hrc_isrgb(t)) src.text_colour = (hrc_ishex(t) ? t : hrc_rgb2hex(t))
	else src << "\"[t]\" is not a valid colour code."

//hrc_rgb2hex(rgb code) \
Converts an RGB decimal code into a "full" seven-character hexadecimal code. Wherever RGB \
decimals are used, they're typically not useful as-is, so the usefulness of this proc \
is obvious.

mob/verb/give_hex(t as text)
	var/hex = hrc_rgb2hex(t)
	if(!hex) src << "\"[t]\" is not a valid RGB decimal."
	else src << "\"[t]\" as a hexadecimal code is \"[hex]\"."

/*Version history

Version 0 (0105.04)
  -Initial release.
Version 1 (0105.04)
  -Made a couple basic changes and added my email to the comment block.
Version 2 (0205.04)
  -Fixed a couple of issues that slipped my mind thanks to Flick;
    -Made sure one couldn't enter a negative number into the rgb2hex() proc.
    -The "#" in a hex string is only allowed as the first character.
  -Messed with the ishex() proc a bit.
  -Shuffled up hex2rgb() some.
  -Fixed a copytext problem within rgb2hex() that cut the RGB values too short.
Version 3 (0505.04)
  -Cleaned up the comment block into a readable state.
Version 4 (1006.04)
  -Because of the support for shorthand hex values (IE, #FFF or #BF0), it is imperative that
  the # is included when a hexadecimal value is required.
Version 5 (0807.04)
  -The value sent through rgb2hex() is now a single text string. It's broken up and parsed
  correctly for you within the proc, assuming it was valid to begin with.
  -Now using lower text names for the procs, so they fit in better with all of DM's built-in ones.
  -You now have to use a control var to determine if output is actually valid. All provided procs
  return null if there is a problem, so you have to make sure there's actual output before
  assigning any values or manipulating anything.
Version 6 (0907.04)
  -rgb2hex() now returns just a shorthand value in the case there is one, instead of
  "#aabbcc or #abc"; I noticed this created a small problem when I was trying to assign text colours
  in my game where an RGB value is valid input.
Version 7 (0509.04)
  -Included the ability for hex2rgb() to return a list containing the RGB data. I realized this
  feature is sometimes a necessity, as I found out when doing the HTML text fader.
  -Changed how the ishex() proc works a bit.
  -Added an HTML text fader. Consult the file "Help.chm" for usage.
  -Placed an "e_" prefix before all support procs.
  -Arranged all the procs in alphabetical order... it's my latest organizational kick.
  -Removed all the comments until I find the energy to make new ones, as the existing ones did
  not reflect the actual functions of the procs.
Version 8 (0609.04)
  -Added a new set of checks to ishex().
  -Went over all the procs and made sure they all supported shorthand hex values. But as
  always, any instance that requires a hexadecimal code requires the #.
Version 9 (0709.04)
  -Messed around with ishex() yet again.
  -Made a few minor optimizations.
  -The procs are actually arranged alphabetically this time. Honest.
  -Revised the comment block.
  -Put the operations of the e_converttohex() proc back into rgb2hex().
  -Cleaned up fadetext() a bit.
Version 10 (1409.04)
  -Removed all instances of "dec" and replaced them with "rgb". This should be the last major
  functionality change.
  -Changed the procs a bit so those that require a true/false value can recieve any conceivable
  true/false value.
  -Found out the actual spelling is "hexadecimal"... so naturally I corrected it anywhere it
  appeared.
  -You can now pass a list of RGB values into rgb2hex().
  -Few other things, but I forgot to document them...
Version 11 (0111.04)
  -Messed around with the comment block a bit.
  -Cleaned up fadetext() a lot.
  -Included a .chm helpfile.
  -Changed how ishex() works.
  -fadetext() can now recieve RGB values.
Version 12 (0511.04)
  -Small problem with ishex(); the provided procs in the library were still using the old
  implementation.
  -Renamed hexfilter() to ishex().
  -Redid and revised the history.
Version 13 (2403.05)
  -Complete and total recode of the entire library.
  -Removed the fadetext() proc.
Version 14 (0706.05)
  -Cleaned up rgb2hex() some more.
  -Added another support proc.
Version 15 (0207.05)
  -Merged e_countcommas and e_findchar.
  -Rewrote some of the library to conform to standards set at
  http://www.byondscape.com/ascape.dmb/Gazoot.2003-0116
Version 16 (2303.06)
  -Unknown Person informed me that the rgb() proc returns a hex value. Naturally I was amazed by
  what I didn't know, and changed the hrc_rgb2hex() proc to simply return the value given by
  rgb().
  -I moved the contents of the CHM file back into a comment block within the code file(obviously).
  -hrc_isrgb() can now accept a list as well as text.
  -Re-coded hrc_isrgb().
Version 17 (0104.06)
  -Added hrc_invert() (inspired by Koil), which inverts the given value (like when you select
  text or images in the Seeker terminal; that's an invert).
  -Added a small bit of explanation at the top, though the function of the procs should be somewhat
  obvious by their name.
Version 18 (0304.06)
  -Added a demo file.
Version 19 (3007.06)
  -RGB values can now use any non-number delimiter.
*/